@@ -33,7 +33,7 @@ algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE)
   #Model formula depends on whether to include a time trend or not.
   theModel <- as.formula(ifelse(timeTrend, "response~1+wtime","response~1"))
 
-  #Fit it.
+  #Fit it -- this is slow. An improvement would be to use glm.fit here.
   model <- glm(theModel, family = quasipoisson(link="log"))
     
  #Check convergence - if no convergence we return empty handed.
@@ -78,7 +78,7 @@ algo.farrington.fitGLM <- function(response,wtime,timeTrend=TRUE,reweight=TRUE)
 ### chunk number 4: 
 ###################################################
 
-algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="none") {
+algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="none",y) {
   #Fetch mu0 and var(mu0) from the prediction object
   mu0 <- pred$fit
   tau <- phi + (pred$se.fit^2)/mu0
@@ -95,18 +95,69 @@ algo.farrington.threshold <- function(pred,phi,alpha=0.01,skewness.transform="no
   #Ensure that lower bound is non-negative
   lu[1] <- max(0,lu[1],na.rm=TRUE)
 
+  #Compute quantiles of the predictive distribution based on the
+  #normal approximation on the transformed scale
+  q <- pnorm( y^(1/exponent) , mean=mu0^exponent, sd=se)
+  m <- qnorm(0.5, mean=mu0^exponent, sd=se)^(1/exponent)
+  
   #Return lower and upper bounds
-  return(lu)
+  return(c(lu,q=q,m=m))
 }
 
 
 ###################################################
 ### chunk number 5: 
 ###################################################
+######################################################################
+# Compute indices of reference value using Date class
+#
+# Params:
+#  t0 - Date object describing the time point
+#  b  - Number of years to go back in time
+#  w  - Half width of window to include reference values for
+#  epochStr - "1 month", "1 week" or "1 day"
+#  epochs - Vector containing the epoch value of the sts/disProg object
+#
+# Details:
+#  Using the Date class the reference values are formed as follows:
+#   Starting from d0 go i, i in 1,...,b years back in time.
+#   
+# Returns:
+#  a vector of indices in epochs which match
+######################################################################
+refvalIdxByDate <- function(t0, b, w, epochStr, epochs) {
+  refDays <- NULL
+  refPoints <- seq( t0, length=b+1, by="-1 year")[-1]
+  for (j in 1:length(refPoints)) {
+    refPointWindow <- c(rev(seq(refPoints[j], length=w+1, by=paste("-",epochStr,sep=""))),
+                        seq(refPoints[j], length=w+1, by=epochStr)[-1])
+    refDays <- append(refDays,refPointWindow)
+  }
+  if (epochStr == "1 week") {
+    #By convention: always go back to the closest monday 
+    #(but always back, never ahead!)
+    refDays <- refDays -  (as.numeric(format(refDays, "%w")) - 1)
+  }
+  if (epochStr == "1 month") {
+    #By convention: go back in time to  closest 1st of month
+    refDays <- refDays -  (as.numeric(format(refDays, "%d")) - 1)
+  }
+  #Find the index of these reference values
+  wtime <- match(as.numeric(refDays), epochs)
+  return(wtime)
+} 
+
+
+
+###################################################
+### chunk number 6: 
+###################################################
 algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, reweight=TRUE, verbose=FALSE,alpha=0.01,trend=TRUE,limit54=c(5,4),powertrans="2/3")) { 
   #Fetch observed
   observed <- disProgObj$observed
   freq <- disProgObj$freq
+  epochStr <- switch( as.character(freq), "12" = "1 month","52" =  "1 week","365" = "1 day")
+
 
   ######################################################################
   # Fix missing control options
@@ -123,7 +174,13 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
   if (is.null(control$plot))     {control$plot=FALSE}
   if (is.null(control$limit54))  {control$limit54=c(5,4)}
   if (is.null(control$powertrans)){control$powertrans="2/3"}
-
+  #Use special Date class mechanism to find reference months/weeks/days
+  if (is.null(disProgObj[["epochAsDate",exact=TRUE]])) { 
+    epochAsDate <- FALSE 
+  } else { 
+    epochAsDate <- disProgObj[["epochAsDate",exact=TRUE]] 
+  }
+    
   #check options
   if (!((control$limit54[1] >= 0) &  (control$limit54[2] > 0))) {
     stop("The limit54 arguments are out of bounds: cases >= 0 and period > 0.")
@@ -133,38 +190,56 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
   alarm <- matrix(data = 0, nrow = length(control$range), ncol = 1)
   trend <- matrix(data = 0, nrow = length(control$range), ncol = 1)
   upperbound <- matrix(data = 0, nrow = length(control$range), ncol = 1)
-  
+  # predictive distribution
+  pd <- matrix(data = 0, nrow = length(control$range), ncol = 2)
+
   # Define objects
   n <- control$b*(2*control$w+1)
   
   # 2: Fit of the initial model and first estimation of mean and dispersion
   #    parameter
-  for(k in control$range) {
+  for (k in control$range) {
     # transform the observed vector in the way
     # that the timepoint to be evaluated is at last position
     #shortObserved <- observed[1:(maxRange - k + 1)]
 
     if (control$verbose) { cat("k=",k,"\n")}
 
-    #Find all weeks with index k-w,..,k+w 
+    #Find index of all epochs, which are to be used as reference values
+    #i.e. with index k-w,..,k+w 
     #in the years (current year)-1,...,(current year)-b
-    wtime <- NULL
-    for (i in control$b:1){
-      wtime <- append(wtime,seq(k-freq*i-control$w,k-freq*i+control$w,by=1))
-    }
-    response <- NULL # die Responsespalte
-    for (i in (control$b:1)) {
-      if (control$verbose) {cat("b=",i,"\trange=",((k-i*freq)-control$w):((k-i*freq)+control$w),"\n")}
-
-      for (j in (((k-i*freq)-control$w):((k-i*freq)+control$w))){
-        if (j<1) {
-          cat("Warning: Selection index less than 1!\n")
-        }
-        else {
-          response <- append(response,observed[j])
-        }
+    if (!epochAsDate) {
+      wtimeAll <- NULL
+      for (i in control$b:1){
+        wtimeAll <- append(wtimeAll,seq(k-freq*i-control$w,k-freq*i+control$w,by=1))
       }
-    } 
+      #Select them as reference values
+      wtime <- wtimeAll[wtimeAll>0]
+      if (length(wtimeAll) != length(wtime)) {
+        warning("Some reference values did not exist (index<1).")
+      }
+    } else { #Alternative approach using Dates
+      t0 <- as.Date(disProgObj$week[k], origin="1970-01-01")
+      wtime <- refvalIdxByDate( t0=t0, b=control$b, w=control$w, epochStr=epochStr, epochs=disProgObj$week)
+    }
+    
+    #Extract values from indices
+    response <- observed[wtime]
+
+##     response <- NULL # die Responsespalte
+##     for (i in (control$b:1)) {
+##       if (control$verbose) {cat("b=",i,"\trange=",((k-i*freq)-control$w):((k-i*freq)+control$w),"\n")}
+
+##       for (j in (((k-i*freq)-control$w):((k-i*freq)+control$w))){
+##         if (j<1) {
+##           cat("Warning: Selection index less than 1!\n")
+##         }
+##         else {
+##           response <- append(response,observed[j])
+##         }
+##       }
+##     } 
+
     if (control$verbose) { print(response)}
 
     ######################################################################
@@ -214,7 +289,7 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     pred <- predict.glm(model,data.frame(wtime=c(k)),dispersion=model$phi,
                         type="response",se.fit=TRUE)
     #Calculate lower and upper threshold
-    lu <- algo.farrington.threshold(pred,model$phi,skewness.transform=control$powertrans,alpha=control$alpha)
+    lu <- algo.farrington.threshold(pred,model$phi,skewness.transform=control$powertrans,alpha=control$alpha, observed[k])
 
     ######################################################################
     # If requested show a plot of the fit.
@@ -229,9 +304,8 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
       #Add the prediction
       lines(data$wtime,preds,col=1,pch=2)
 
-      #Add the thresholds
-      #points(c(k,k),lu,cex=1,pch=3,col=3)
-      lines(rep(k,2),lu,col=3,lty=2)
+      #Add the thresholds to the plot
+      lines(rep(k,2),lu[1:2],col=3,lty=2)
     }
 
 
@@ -247,18 +321,23 @@ algo.farrington <- function(disProgObj, control=list(range=NULL, b=3, w=3, rewei
     #the okHistory variable meant to protect against zero count problems,
     #but instead it resulted in exceedance score == 0 for low counts. 
     #Now removed to be concordant with the Farrington 1996 paper.
-    X <- ifelse(enoughCases,(observed[k] - pred$fit) / (max(lu) - pred$fit),0)
+    X <- ifelse(enoughCases,(observed[k] - pred$fit) / (lu[2] - pred$fit),0)
 
     #Do we have an alarm -- i.e. is observation beyond CI??
     #upperbound only relevant if we can have an alarm (enoughCases)
     trend[k-min(control$range)+1] <- doTrend
     alarm[k-min(control$range)+1] <- (X>1)
     upperbound[k-min(control$range)+1] <- ifelse(enoughCases,lu[2],0)
+    #Compute bounds of the predictive
+    pd[k-min(control$range)+1,] <- lu[c(3,4)]
+
   }#done looping over all time points
 
   #Add name and data name to control object.
   control$name <- paste("farrington(",control$w,",",0,",",control$b,")",sep="")
   control$data <- paste(deparse(substitute(disProgObj)))
+  #Add information about predictive distribution
+  control$pd   <- pd
 
   # return alarm and upperbound vectors 
   result <- list(alarm = alarm, upperbound = upperbound, trend=trend, 
